@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 // Package datachannel implements WebRTC Data Channels
 package datachannel
 
@@ -32,6 +35,11 @@ type Writer interface {
 	WriteDataChannel([]byte, bool) (int, error)
 }
 
+// WriteDeadliner extends an io.Writer to expose setting a write deadline.
+type WriteDeadliner interface {
+	SetWriteDeadline(time.Time) error
+}
+
 // ReadWriteCloser is an extended io.ReadWriteCloser
 // that also implements our Reader and Writer.
 type ReadWriteCloser interface {
@@ -42,6 +50,14 @@ type ReadWriteCloser interface {
 	io.Closer
 }
 
+// ReadWriteCloserDeadliner is an extended ReadWriteCloser
+// that also implements r/w deadline.
+type ReadWriteCloserDeadliner interface {
+	ReadWriteCloser
+	ReadDeadliner
+	WriteDeadliner
+}
+
 // DataChannel represents a data channel
 type DataChannel struct {
 	Config
@@ -71,12 +87,12 @@ type Config struct {
 	LoggerFactory        logging.LoggerFactory
 }
 
-func newDataChannel(stream *sctp.Stream, config *Config) (*DataChannel, error) {
+func newDataChannel(stream *sctp.Stream, config *Config) *DataChannel {
 	return &DataChannel{
 		Config: *config,
 		stream: stream,
 		log:    config.LoggerFactory.NewLogger("datachannel"),
-	}, nil
+	}
 }
 
 // Dial opens a data channels over SCTP
@@ -115,7 +131,7 @@ func Client(stream *sctp.Stream, config *Config) (*DataChannel, error) {
 			return nil, fmt.Errorf("failed to send ChannelOpen %w", err)
 		}
 	}
-	return newDataChannel(stream, config)
+	return newDataChannel(stream, config), nil
 }
 
 // Accept is used to accept incoming data channels over SCTP
@@ -164,10 +180,7 @@ func Server(stream *sctp.Stream, config *Config) (*DataChannel, error) {
 	config.Label = string(openMsg.Label)
 	config.Protocol = string(openMsg.Protocol)
 
-	dataChannel, err := newDataChannel(stream, config)
-	if err != nil {
-		return nil, err
-	}
+	dataChannel := newDataChannel(stream, config)
 
 	err = dataChannel.writeDataChannelAck()
 	if err != nil {
@@ -224,6 +237,12 @@ func (c *DataChannel) SetReadDeadline(t time.Time) error {
 	return c.stream.SetReadDeadline(t)
 }
 
+// SetWriteDeadline sets a deadline for writes to return,
+// only available if the BlockWrite is enabled for sctp
+func (c *DataChannel) SetWriteDeadline(t time.Time) error {
+	return c.stream.SetWriteDeadline(t)
+}
+
 // MessagesSent returns the number of messages sent
 func (c *DataChannel) MessagesSent() uint32 {
 	return atomic.LoadUint32(&c.messagesSent)
@@ -280,13 +299,12 @@ func (c *DataChannel) handleDCEP(data []byte) error {
 
 	switch msg := msg.(type) {
 	case *channelAck:
-		c.log.Debug("Received DATA_CHANNEL_ACK")
-		if err = c.commitReliabilityParams(); err != nil {
+		if err := c.commitReliabilityParams(); err != nil {
 			return err
 		}
 		c.onOpenComplete()
 	default:
-		return fmt.Errorf("%w %v", ErrInvalidMessageType, msg)
+		return fmt.Errorf("%w, wanted ACK got %v", ErrUnexpectedDataChannelType, msg)
 	}
 
 	return nil
